import bpy
from bpy import *
from urllib.parse import urlencode
from urllib.request import *
from html.parser import *
from smtplib import *
from email.mime.text import MIMEText
import time, platform, os
import codecs
import bpy.path
from bpy.path import abspath
import base64
import mathutils
from mathutils import *
from math import *
import pickle
import shutil
import subprocess

from .BB267___COMMONS import *
from .BB267___VIEW import *
from os.path import expanduser

bpy.types.Object.BBInfo = bpy.props.StringProperty()




# ========================= EP = Start ===============================================
class bb_operator_ep(types.Operator):
	bl_idname = "ops.bb_operator_ep"
	bl_label = "Show EP"
	bl_description = "Calculate and Visualize Electric Potential"
	def invoke(self, context, event):
		try:
			bpy.context.user_preferences.edit.use_global_undo = False
			cleanEPObjs()
			ep(animation = False)
			bpy.context.scene.BBViewFilter = "4"
			bpy.context.user_preferences.edit.use_global_undo = True
			todoAndviewpoints()
		except Exception as E:
			s = "Generate EP Visualization Failed: " + str(E)
			print(s)
			return {'CANCELLED'}
		else:
			return{'FINISHED'}
bpy.utils.register_class(bb_operator_ep)


def ep(animation, verbose =  True):
	global epOBJ
	scene = bpy.context.scene
	setup(clear=False)

	# make new surface
	if not (animation or select("ShapeIndexedFaceSet")):
		print("Generating Surface")
		surface()
	
	if (not animation) or (bpy.context.scene.frame_current % 5 == 1):
		print("Generating EP Curves")
		# has H?
		#extraCommand = ""
		tmpPathOpen = homePath +  "tmp" + os.sep + "tmp.pdb"
		
		
		with open(tmpPathOpen,"r") as file:
			for line in file:
				line = line.replace("\n", "")
				line = line.replace("\r", "")
				line = PDBString(line)
				tag = line.get("tag")

				# if tag is ATOM, load column data
				if (tag == "ATOM" or tag == "HETATM"):
					# check for element type
					if line.get("element") == H:
						extraCommand = "--assign-only"
						break

		# select the forcefield
		forcefield = bpy.context.scene.BBForceField
		if forcefield == "0": method = "amber"
		elif forcefield == "1": method = "charmm"
		elif forcefield == "2": method = "parse"
		elif forcefield == "3": method = "tyl06"
		elif forcefield == "4": method = "peoepb"
		elif forcefield == "5": method = "swanson"



		print("Running PDB2PQR")
		if linux:
			os.chdir(quotedPath(homePath + "bin" + os.sep + "pdb2pqr-1.6" + os.sep))
		else:
			os.chdir(r"\\?\\" + homePath + "bin" + os.sep + "pdb2pqr-1.6" + os.sep)
		tmpPathPar1 = "python"
		tmpPathPar2 = homePath + "bin" + os.sep + "pdb2pqr-1.6" + os.sep + "pdb2pqr.py"
		tmpPathPar3 = homePath + "tmp" + os.sep + "tmp.pqr"
		tmpPathPar4 = homePath + "tmp" + os.sep + "tmp.pdb"
		if linux:
			command = "%s %s --apbs-input --ff=%s %s %s" % (tmpPathPar1, tmpPathPar2, method, tmpPathPar4, tmpPathPar3)
		else:
			command = "%s %s --apbs-input --ff=%s %s %s" % (quotedPath(tmpPathPar1), quotedPath(tmpPathPar2), method, quotedPath(tmpPathPar4), quotedPath(tmpPathPar3))
		launch(exeName=command)




		print("Running inputgen.py")
		tmp1PathPar1 = "python"
		tmp1PathPar2 = homePath + "bin" + os.sep + "pdb2pqr-1.6" + os.sep + "src" + os.sep + "inputgen.py"
		tmp1PathPar3 = homePath + "tmp" + os.sep + "tmp.pqr"
		if linux:
			command = "%s %s --istrng=%f --method=auto --space=%f %s" % (tmp1PathPar1, tmp1PathPar2, bpy.context.scene.BBEPIonConc, bpy.context.scene.BBEPGridStep, tmp1PathPar3)
		else:
			command = "%s %s --istrng=%f --method=auto --space=%f %s" % (quotedPath(tmp1PathPar1), quotedPath(tmp1PathPar2), bpy.context.scene.BBEPIonConc, bpy.context.scene.BBEPGridStep, quotedPath(tmp1PathPar3))
		launch(exeName=command)


		print("Running APBS")
		try:
			if linux:
				shutil.copy(quotedPath(homePath + "bin" + os.sep + "apbs-1.2.1" + os.sep + "apbs.exe"), quotedPath(homePath + "tmp" + os.sep + "apbs.exe"))
			else:
				shutil.copy(r"\\?\\" + homePath + "bin" + os.sep + "apbs-1.2.1" + os.sep + "apbs.exe", r"\\?\\" + homePath + "tmp" + os.sep + "apbs.exe")
		except Exception as E:
			s = "APBS COPY failed: " + str(E)
			print(s)
		if linux:
			oPath = homePath + "tmp" + os.sep + "tmp.in"
			f = open(oPath, "r")
			lines = f.readlines()
			f.close()
			lines[1] = "    mol pqr " + quotedPath(homePath + "tmp" + os.sep + "tmp.pqr") + "\n"
			f = open(oPath, "w")
			f.writelines(lines)
			f.close()
			command = "chmod 755 %s" % (quotedPath(homePath + "tmp" + os.sep + "apbs.exe"))
			command = quotedPath(command)
			launch(exeName = command)
			command = homePath + "tmp" + os.sep + "apbs.exe" + " " + homePath + "tmp" + os.sep + "tmp.in"
		else:
			oPath = homePath + "tmp" + os.sep + "tmp.in"
			f = open(oPath, "r")
			lines = f.readlines()
			f.close()
			lines[1] = "    mol pqr " + quotedPath(homePath + "tmp" + os.sep + "tmp.pqr") + "\n"
			f = open(oPath, "w")
			f.writelines(lines)
			f.close()
			command = quotedPath(homePath + "tmp" + os.sep + "apbs.exe") + " " + quotedPath(homePath + "tmp" + os.sep + "tmp.in")
		p = launch(exeName=command, async = True)
		print("APBS Ok")
		
		# sync
		wait(p)
		
		# write pot dx pot problems: writes the .dx file in user home path...
		print("============ POT DX POT COPY ================")
		envBoolean = False
		try:
			if linux:
				tmpP = quotedPath(homePath + "tmp" + os.sep + "pot.dx")
				if os.path.isfile(tmpP):
					envBoolean = True
					print("pot.dx in current directory; won't search in HOME or VIRTUALSTORE folders...")
		except Exception as E:
			s = "pot.dx output rewrite failed in tmp.in, will search in some folders...: " + str(E)
			print(s)
		if envBoolean == False:
			if linux:
				print("user home: ", os.path.expanduser("~"))
				try:
					print("BB stays here: ")
					homeutente = os.path.expanduser("~")
					shutil.move(quotedPath(homeutente + "/.config/blender/2.67/scripts/addons/BB267/bin/pdb2pqr-1.6/pot.dx"), quotedPath(homeutente + "/.config/blender/2.67/scripts/addons/BB267/tmp/pot.dx"))
					shutil.move(quotedPath(homeutente + "/.config/blender/2.67/scripts/addons/BB267/bin/pdb2pqr-1.6/io.mc"), quotedPath(homeutente + "/.config/blender/2.67/scripts/addons/BB267/tmp/io.mc"))
				except Exception as E:
					s = "pot.dx not found in HOME: " + str(E)
					print(s)
			else:
				try:
					envHome = str(os.environ['USERPROFILE'])
					print("envHome: " + envHome)
					shutil.move(r"\\?\\" + envHome + os.sep + "pot.dx", r"\\?\\" + homePath + "tmp" + os.sep + "pot.dx")
					shutil.move(r"\\?\\" + envHome + os.sep + "io.mc", r"\\?\\" + homePath + "tmp" + os.sep + "io.mc")
					envBoolean = True
				except Exception as E:
					s = "No pot.dx in HOME: " + str(E)
					print(s)
				if not envBoolean:
					print("Win problem; will search in Windows...")
					try:
						envHome = "C:" + os.sep + "Windows"
						print("envHome: " + envHome)
						shutil.move(envHome + os.sep + "pot.dx", homePath + "tmp" + os.sep + "pot.dx")
						shutil.move(envHome + os.sep + "io.mc", homePath + "tmp" + os.sep + "io.mc")
						envBoolean = True
					except Exception as E:
						s = "Windows home failed too; no pot.dx, sorry: " + str(E)
						print(s)
				if not envBoolean:
					print("Win problem; will search in AppData - Local - VirtualStore...")
					try:
						envHome = str(os.environ['USERPROFILE']) + os.sep + "AppData" + os.sep + "Local" + os.sep + "VirtualStore"
						print("envHome: " + envHome)
						shutil.move(envHome + os.sep + "pot.dx", homePath + "tmp" + os.sep + "pot.dx")
						shutil.move(envHome + os.sep + "io.mc", homePath + "tmp" + os.sep + "io.mc")
						envBoolean = True
					except Exception as E:
						s = "VirtualStore failed too; no pot.dx, sorry: " + str(E)
						print(s)
				if not envBoolean:
					print("Win problem; will search in AppData - Local - VirtualStore - Windows...")
					try:
						envHome = str(os.environ['USERPROFILE']) + os.sep + "AppData" + os.sep + "Local" + os.sep + "VirtualStore" + os.sep + "Windows"
						print("envHome: " + envHome)
						shutil.move(envHome + os.sep + "pot.dx", homePath + "tmp" + os.sep + "pot.dx")
						shutil.move(envHome + os.sep + "io.mc", homePath + "tmp" + os.sep + "io.mc")
						envBoolean = True
					except Exception as E:
						s = "VirtualStore - Windows failed too; no pot.dx, sorry: " + str(E)
						print(s)
						print("=========== SORRY: CANNOT FIND POT.DX ============")


		print ("Saving obj")
		exportOBJ(homePath +"tmp" + os.sep + "tmp")

		if len(epOBJ) >= maxCurveSet:
			# delete the oldest curve-sets out of the list.
			epOBJ.reverse()
			deletionList = epOBJ.pop()			
			cleanEPObjs(deletionList)
			epOBJ.reverse()
			
		print("Running Scivis")
		if linux:
			command = "chmod 755 %s" % (quotedPath(homePath + "bin" + os.sep + "scivis" + os.sep + "SCIVIS.exe"))
			command = quotedPath(command)
			launch(exeName = command)
			command = "%s %s %s %s %f %f %f %f %f" % (homePath + "bin" + os.sep + "scivis" + os.sep + "SCIVIS.exe", homePath + "tmp" + os.sep + "tmp.obj", homePath + "tmp" + os.sep + "pot.dx", homePath + "tmp" + os.sep + "tmp.txt", bpy.context.scene.BBEPNumOfLine/10, bpy.context.scene.BBEPMinPot, 45, 1, 3)
		else:
			command = "%s %s %s %s %f %f %f %f %f" % (quotedPath(homePath + "bin" + os.sep + "scivis" + os.sep + "SCIVIS.exe"), quotedPath(homePath + "tmp" + os.sep + "tmp.obj"), quotedPath(homePath + "tmp" + os.sep + "pot.dx"), quotedPath(homePath + "tmp" + os.sep + "tmp.txt"), bpy.context.scene.BBEPNumOfLine/10, bpy.context.scene.BBEPMinPot, 45, 1, 3)
		launch(exeName=command)
		
		print("Importing data into Blender")
		
		list = importEP(homePath + "tmp" + os.sep + "tmp.txt")
		
		epOBJ.append(list)

		ob = select("Emitter")

		print("Current frame before if animation: " + str(bpy.context.scene.frame_current))
		if animation:
			ob.particle_systems[0].settings.frame_start = bpy.context.scene.frame_current
			print("Frame start: " + str(ob.particle_systems[0].settings.frame_start))
			ob.particle_systems[0].settings.frame_end = bpy.context.scene.frame_current
			print("Frame end: " + str(ob.particle_systems[0].settings.frame_end))
			ob.particle_systems[0].settings.frame_end = 20
			print("Lifetime: " + str(ob.particle_systems[0].settings.frame_end))
			ob.particle_systems[0].settings.count = len(ob.data.vertices)
			print("Animation")
		else:
			if not bpy.context.screen.is_animation_playing:
				bpy.ops.screen.animation_play()
		
		# Hide atoms
		try:
			for obj in bpy.context.scene.objects:
				obj.hide = False
				obj.hide_render = False
				obj.draw_type = "TEXTURED"
				if re.search("#",obj.name):
					line=obj.BBInfo
					line = PDBString(line)
					elementName = line.get("element")
					atomName = line.get("name")
					obj.hide = True
					obj.hide_render = True
		except Exception as E:
			s = "Hiding atoms failed: " + str(E)
			print(s)


# import curve description text into Blender
def importEP(path):
	global curveCount
	global objList
	
	curveCount = 0
	scene = bpy.context.scene
	pts = []
	objList = []
	# read the file once to generate curves
	with open(path,"r") as file:
		for file_line in file:
			line=file_line.split()
			if line[0]=="n":
				if curveCount !=0:
					# for every n encountered creates a new curve
					cu = bpy.data.curves.new("Curve%3d"%curveCount, "CURVE")
					ob = bpy.data.objects.new("CurveObj%3d"%curveCount, cu)
					bpy.context.scene.objects.link(ob)
					bpy.context.scene.objects.active = ob
					# set all the properties of the curve
					spline = cu.splines.new("NURBS")
					cu.dimensions = "3D"
					cu.use_path = True
					cu.resolution_u = 1
					spline.points.add(len(pts)//4-1)
					spline.points.foreach_set('co', pts)
					spline.use_endpoint_u = True
					ob.field.type="GUIDE"
					ob.field.use_max_distance=True
					ob.field.distance_max=0.05
					# objList keeps a list of all EP related objects for easy deletion
					objList.append(ob)
					pts = []
				curveCount += 1
			elif line[0]=="v":
				pts.append(float(line[1]))
				pts.append(float(line[2]))
				pts.append(float(line[3]))
				pts.append(1)

	# rename current emitter
	if select("Emitter"):
		for list in epOBJ:
			for obj in list:
				if obj.name == "Emitter":
					obj.name = "Emitter%d"%curveCount

	# read the file again to generate the particle emitter object
	with open(path,"r") as file:
		verts=[]
		for line in file:
			# read the first line after each 'n' identifier
			if "n" in line:
				next = file.readline()
				coord = next.split()
				verts.append([float(i) for i in coord[1:]])

		# make mesh
		mesh = bpy.data.meshes.new("Emitter")
		mesh.from_pydata(verts[:-1], [], [])		# [:-1] to fix the off by one error somewhere...

		# append emitter object with particle sytem into scene, and assign mesh to the object
		# this is a workaround to avoid having to add a particle system from the scene context (impossible)
		Directory = homePath + "library.blend" + os.sep + "Object" + os.sep
		Path = os.sep + os.sep + "library.blend" + os.sep + "Object" + os.sep + "Emitter"
		objName = "Emitter"
		bpy.ops.wm.link_append(filepath = Path, filename = objName, directory = Directory, link = False)
		
		ob = bpy.data.objects["Emitter"]
		print("EMITTER in ob: " + ob.name)
		ob.data = mesh
		
		# 2013-06-27 - Particle Settings End Frame
		#try:
		#	bpy.data.particles['ParticleSettings'].frame_end = bpy.context.scene.frame_end
			#bpy.data.particles['ParticleSettings'].frame_end = bpy.context.scene.frame_end
			#lifetime
		#except Exception as E:
		#	s = "Emitter Particle System frame_end NOT SET: " + str(E)
		#	print(s)

		# add material if does not exist
		if not ob.data.materials:
			mat = bpy.data.materials["Particles"]
			ob.data.materials.append(mat)

		# change particle density according to curve count
		ob.particle_systems[0].settings.count = int(bpy.context.scene.BBEPNumOfLine * 50000 * bpy.context.scene.BBEPParticleDensity)
		# reset location
		ob.location = [0,0,0]
		# add object to deletion list
		objList.append(ob)
	return objList


# Convert WRL to OBJ for scivis.exe
def exportOBJ(path):
	vertexData = []		# list of list[3] (wrl vertices data)
	# read wrl file
	with open(path+".wrl") as wrl:
		found = False
		for line in wrl:
			# skip to coord section of the file
			if not found:
				if "coord" in line:
					wrl.readline()
					found = True
			# when in the coord section of the file
			else:
				if "]" not in line:
					# convert vertexData from string to a list of float
					entry = line[:-2].split()
					entryFloat = [float(coord) for coord in entry]
					vertexData.append(entryFloat)
				else:
					# end document processing
					break

	# write obj file: vertex data
	with open(path+".obj", mode="w") as obj:
		for entry in vertexData:
			out = "v %f %f %f\n" % (entry[0], entry[1], entry[2])
			obj.write(out)

		# face data
		i = 0
		while (i < len(vertexData)):
			out = "f %d/%d %d/%d %d/%d\n" % (i+1,i+1,i+2,i+2,i+3,i+3)
			obj.write(out)
			i = i + 3
# ========================= EP = End ===============================================


# ===================== EP CLEAR = Start ===========================================
class bb_operator_ep_clear(types.Operator):
	bl_idname = "ops.bb_operator_ep_clear"
	bl_label = "Clear EP"
	bl_description = "Clear the EP Visualization"
	def invoke(self, context, event):
		try:
			bpy.context.user_preferences.edit.use_global_undo = False
			cleanEPObjs()
			bpy.context.user_preferences.edit.use_global_undo = True
			todoAndviewpoints()
		except Exception as E:
			s = "Clear EP Visualization Failed: " + str(E)
			print(s)
			return {'CANCELLED'}
		else:
			return{'FINISHED'}
bpy.utils.register_class(bb_operator_ep_clear)


# delete EP related objects
def cleanEPObjs(deletionList = None):
	global epOBJ
	bpy.ops.object.select_all(action="DESELECT")
	# use deletionList if supplied
	if deletionList:
		for obj in deletionList:
			obj.select = True
	# otherwise delete everything in EPOBJ list
	else:
		for list in epOBJ:
			for obj in list:
				obj.select = True
		epOBJ = []
	# call delete operator
	bpy.ops.object.delete()
# ===================== EP CLEAR = End ===========================================


if __name__ == "__main__":
	print("EP module created")