import bpy
from bpy import *
from urllib.parse import urlencode
from urllib.request import *
from html.parser import *
from smtplib import *
from email.mime.text import MIMEText
import time, platform, os
import codecs
import bpy.path
from bpy.path import abspath
import base64
import mathutils
from mathutils import *
from math import *
import pickle
import shutil
import subprocess
import sys
import copy


from .BB267___COMMONS import *
from .BB267___GE import *
from .BB267___VIEW import *
from .BB267___PDBimport import *
from .BB267___EP import *
from .BB267___MLP import *
from .BB267___NMA import *


bpy.types.Object.BBInfo = bpy.props.StringProperty()


class bb_operator_export_pdb(types.Operator):
	bl_idname = "ops.bb_operator_export_pdb"
	bl_label = "Export PDB"
	bl_description = "Export current view to PDB"
	def invoke(self, context, event):
		try:
			bpy.context.user_preferences.edit.use_global_undo = False
			exportSequence(PDBOnly = True)
			bpy.context.user_preferences.edit.use_global_undo = True
		except Exception as E:
			s = "Export PDB Failed: " + str(E)
			print(s)
			return {'CANCELLED'}
		else:
			return{'FINISHED'}
bpy.utils.register_class(bb_operator_export_pdb)


class bb_operator_anim(types.Operator):
	bl_idname = "ops.bb_operator_anim"
	bl_label = "Export Movie"
	bl_description = "Make a movie"
	def invoke(self, context, event):
		try:
			bpy.context.user_preferences.edit.use_global_undo = False
			exportSequence()
			bpy.context.user_preferences.edit.use_global_undo = True
		except Exception as E:
			s = "Export Movie Failed: " + str(E)
			print(s)
			return {'CANCELLED'}
		else:
			return{'FINISHED'}
bpy.utils.register_class(bb_operator_anim)


def exportSequence(verbose =  True, PDBOnly = False):
	step = bpy.context.scene.BBExportStep
	start = bpy.context.scene.frame_start
	end = bpy.context.scene.frame_end
	renderMode = bpy.context.scene.BBRender
	bpy.context.scene.render.engine = 'BLENDER_RENDER'
	
	a = time.time()
	if not modelContainer:
		raise Exception("No PDB file is loaded into Blender, cannot export anything.")

	pdbPath = os.path.normpath(abspath(bpy.context.scene.render.filepath))

	if linux:
		if not os.path.isdir(pdbPath):
			os.mkdir(abspath(pdbPath))
	else:
		if not os.path.isdir(r"\\?\\" + pdbPath):
			os.mkdir(r"\\?\\" + pdbPath)

	i = start
	while i <= end:
		bpy.context.scene.frame_start = start
		bpy.context.scene.frame_end = end
		bpy.context.scene.frame_set(i)
		current_frame = bpy.context.scene.frame_current
		if PDBOnly:
			# export PDB file only
			exportPDB(path = pdbPath+"/%s%d.pdb" % (activeTag, i), tag = activeTag, verbose = True)
		else:
			if renderMode == "0":
				#print ("Rendering atoms")
				bpy.context.scene.BBViewFilter = "3"
				updateView()
			elif renderMode == "1":
				#print ("Rendering Surface")
				setup()
				surface()
				bpy.context.scene.BBViewFilter = "4"
				updateView()
			elif renderMode == "2":
				#print ("Rendering MLP")
				mlp(force = True)
				mlpRender()
				bpy.context.scene.BBViewFilter = "4"
				updateView()
			elif renderMode == "3":
				#bpy.context.scene.frame_set(1)
				#print ("Rendering EP")
				setup()
				surface()
				ep(animation=True)
				bpy.context.scene.BBViewFilter = "4"
				#bpy.context.scene.frame_set(i)
				updateView()
				step = 1  # EP animation should not skip steps, due to particles behavior
			elif renderMode == "4":
				#print ("Rendering EP+MLP")
				mlp(force = True)
				mlpRender()
				#bpy.context.scene.frame_set(1)
				ep(animation=True)
				#bpy.context.scene.frame_set(i)
				bpy.context.scene.BBViewFilter = "4"
				updateView()
				step = 1   # EP animation should not skip steps, due to particles behavior
			# render frame
			current_frame = i
			bpy.context.scene.frame_current = 1
			bpy.context.scene.frame_start = i
			bpy.context.scene.frame_end = i
			bpy.context.scene.frame_current = i
			bpy.ops.render.render(animation=True)
		i += step
	# clean up
	bpy.context.scene.frame_set(start)
	bpy.context.scene.frame_start = start
	bpy.context.scene.frame_end = end
	print(time.time()-a)



if __name__ == "__main__":
	print("OUTPUT module created")