import bpy
from bpy import *
from urllib.parse import urlencode
from urllib.request import *
from html.parser import *
from smtplib import *
from email.mime.text import MIMEText
import time, platform, os
import codecs
import bpy.path
from bpy.path import abspath
import base64
import mathutils
from mathutils import *
from math import *
import pickle
import shutil
import subprocess


from .BB267___COMMONS import *


class bb_operator_import(types.Operator):
	bl_idname = "ops.bb_operator_import"
	bl_label = "Import PDB"
	bl_description = "generate 3D Model"
	def invoke(self, context, event):
		try:
			context.user_preferences.edit.use_global_undo = False
			core_importFile()
			context.user_preferences.edit.use_global_undo = True
		except Exception as E:
			s = "Import Failed: " + str(E)
			print (s)
			return {'CANCELLED'}
		else:
			return{'FINISHED'}
bpy.utils.register_class(bb_operator_import)


class ReImportFileDialogOperator(bpy.types.Operator):
	bl_idname = "object.reimport_file_dialog_operator"
	bl_label = "File already imported; import again?"
	def execute(self, context): 
		core_parsePDB(filePath)
		return {'FINISHED'} 
	def invoke(self, context, event): 
		return context.window_manager.invoke_props_dialog(self) 
bpy.utils.register_class(ReImportFileDialogOperator)


# retrieve PDB from pdb.org
def pdbdotorg(id):
	url1 = str("http://www.pdb.org/pdb/files/"+id+".pdb")
	
	save1 = str(homePath+"fetched" + os.sep + id +".pdb")
	
	if linux:
		if not os.path.isdir(quotedPath(homePath+"fetched")):
			os.mkdir(quotedPath(homePath + "fetched"))
	else:
		if not os.path.isdir(r"\\?\\" + homePath+"fetched"):
			os.mkdir(r"\\?\\" + homePath+"fetched")
	# get file from the web
	try:
		filename, header = urlretrieve(url1, save1)
		bpy.context.scene.BBImportPath = save1
		importPreview(False, True)
		return filename
	except:
		return False


def importPreview(verbose =  False, retrieved = False):
	global currentModelID
	global model
	global filePath
	filePath = bpy.context.scene.BBImportPath
	filePath = abspath(filePath)
	# get PDB straight from PDB.org
	if (len(filePath)==4) and not(retrieved):
		retrivedFile = pdbdotorg(filePath)
		if retrivedFile:
			bpy.context.scene.BBImportFeedback = "Found matching Protein on PDB.org"
		else:
			bpy.context.scene.BBImportFeedback = "Nothing matching this ID found on PDB.org"
			return False
	extension = filePath.lower().endswith
	try:
		file = open(filePath,"r")
	except:
		bpy.context.scene.BBImportFeedback = "File not found"
		return False
	else:
		if extension(".pdb") or extension(".pqr") or extension(".txt"):
			bpy.context.scene.BBModelRemark = bpy.context.scene.BBImportPath[-8:-4]
		else:
			bpy.context.scene.BBModelRemark = bpy.context.scene.BBImportPath[-4:]
		# open is successful, check file extension
		if extension(".pdb") or extension(".pqr"):
			modelCount = 0
			importFileModel = []
			# read the chain id
			importChainID = []
			# read file
			for line in file:
				line = PDBString(line)
				# count number of models in file using the start tag MODEL
				if line.get("tag") == "MODEL":
					currentModelID = line.get("modelID")
					importFileModel.append(currentModelID)
				# count number of models in file using the end tag MODEL
				elif line.get("tag") == "ENDMDL":
					modelCount += 1
				# count number of chains in file using the start tag ATOM    1
				if line.get("tag") == "ATOM":
					currentChainID = line.get("chainID")
					chainCount = 0
					for elem in importChainID:
						if elem == currentChainID:
							chainCount += 1
					if chainCount == 0:
						importChainID.append(currentChainID)
			# Special case for files containing 1 model with no opening model tag
			if len(importFileModel) == 0: importFileModel.append(0)
			# show list of models for importer to load
			bpy.context.scene.BBImportOrder = str(importFileModel)[1:-1]
			# if all okay, display feedback message
			bpy.context.scene.BBImportFeedback = "File contains " + str(len(importFileModel)) + " PDB Models"
			# if all okay, display feedback message
			bpy.context.scene.BBImportChain = "File contains chains " + str(importChainID)
			file.close()
			return True
		if extension(".txt") or extension(".csv"):
			# simply get list of all files from a th specified directory
			fileList = os.listdir(os.path.dirname(filePath))
			fileListClean = []
			for f in fileList:
				# find only matching extensions
				ext = f[-4:].lower()
				if  ext == '.txt' or ext == '.csv':
					fileListClean.append (f)
			bpy.context.scene.BBImportOrder = ", ".join(fileListClean)
			# if all okay, display feedback message
			bpy.context.scene.BBImportFeedback = "Folder contains " + str(len(fileListClean)) + " text Files"
			file.close()
			return True
		# otherwise...
		bpy.context.scene.BBImportFeedback = "Error: Unrecognized File Extension"
		file.close()
		return False


# lookup sidechain parents
def bondLookUp(atom, amac):
	if(atom=="O" or atom=="OXT"):	parent = ["C",C]
	elif(atom=="CB" or atom=="HA" or atom=="HA2" or atom=="HA3"):	parent = ["CA",C]
	elif(atom=="SG" or "HB" in atom or "OG" in atom or "CG" in atom):	parent = ["CB",C]
	elif(atom=="H" or atom=="H1" or atom=="H2" or atom=="H3"):	parent = ["N",N]
	elif(atom=="HG1"):	parent = ["OG1",O]
	elif(atom=="HG23" or atom=="HG22" or atom=="HG21"):	parent = ["CG2",C]
	elif(atom=="SD" or "CD" in atom or "ND" in atom or atom=="HG2" or atom=="HG3" or atom=="OD1" or atom=="OD2" or atom=="HG12" or atom=="HG13" or atom=="HG13"):
			if(amac=="ILE" or amac=="VAL"):	parent = ["CG1",C]
			else:	parent = ["CG",C]
	elif(atom=="CE2" or atom=="CE3" or atom=="NE2" or atom=="HD2"):
			if(amac=="GLN"):	parent = ["CD",C]
			elif(amac=="ARG" or amac=="LYS" or amac=="PRO"):	parent = ["CD",C]
			elif(amac=="ASP"):	parent = ["OD2",O]
			else:	parent = ["CD2",C]
	elif(atom=="CE1" or atom=="HD11" or atom=="HD12" or atom=="HD13" or atom=="HD1" or atom=="NE1"):
			if(amac=="HIS"):	parent = ["ND1",N]
			else:	parent = ["CD1",C]
	elif(atom=="NE" or atom=="HD3" or atom=="CE" or atom=="OE1" or atom=="OE2"):
			if(amac=="MET"):	parent = ["SD",S]
			else:	parent = ["CD",C]
	elif(atom=="CZ" or atom=="HE" or atom=="HE1"):
			if(amac=="ARG"):	parent = ["NE",N]
			elif(amac=="TRP"):	parent = ["NE1",N]
			elif(amac=="MET"):	parent = ["CE",C]
			elif(amac=="PHE" or amac=="HIS" or amac=="TYR"):		parent = ["CE1",C]
	elif(atom=="NH1" or atom=="NH2" or atom=="HZ" or atom=="OH"):		parent = ["CZ",C]
	elif(atom=="HH11" or atom=="HH12" or atom=="1HH1" or atom=="1HH2"):	parent = ["NH1",N]
	elif(atom=="HH21" or atom=="HH22" or atom=="2HH2" or atom=="1HH2"):	parent = ["NH2",N]
	elif(atom=="HD21" or atom=="HD22" or atom=="HD23"):
			if(amac=="LEU"):	parent = ["CD2",C]
			else:	parent = ["ND2",N]
	elif(atom=="HE3"  or atom=="NZ"):
			if(amac=="TRP"):	parent = ["CE3",C]
			else:	parent = ["CE",C]
	elif(atom=="HZ1" or atom=="HZ2" or atom=="HZ3"):
			if(amac=="TRP" and atom=="HZ2"):	parent = ["CZ2",S]
			elif(amac=="TRP" and atom=="HZ3"):	parent = ["CZ3",S]
			else:	parent = ["NZ",N]
	elif(atom=="HG"):
			if(amac=="LEU"):	parent = ["CG",C]
			if(amac=="CYS"):	parent = ["SG",S]
			else:	parent = ["OG",O]
	elif(atom=="HE2" or atom=="CZ2" or atom=="HE21" or atom=="HE22"):
			if(amac=="HIS" or amac=="GLN"):	parent = ["NE2",N]
			elif(amac=="PHE" or amac=="TYR" or amac=="TRP"):	parent = ["CE2",C]
			elif(amac=="GLU"):	parent = ["OE2",O]
			elif(amac=="MET" or amac=="LYS"):	parent = ["CE",C]
	elif(atom=="HH"):	parent = ["OH",O]
	elif(atom=="CZ3"):	parent = ["CE3",C]
	elif(atom=="CH2"):	parent = ["CZ2",C]
	elif(atom=="HH2"):	parent = ["CH2",C]
	return parent


def create_pdb_empty_set():
	bpy.context.scene.layers[19] = True
	for i in range(0,19):
		bpy.context.scene.layers[i] = False
	bpy.context.scene.layers[19] = True
	elementiDaImportare = ['Empty', 'Hemi']
	try:
		for objName in elementiDaImportare:
			Directory = homePath + "EmptySet.blend" + "/" + "Object" + "/"
			Path = "//" + "EmptySet.blend" + "/" + "Object" + "/" + objName
			bpy.ops.wm.link_append(filepath = Path, filename = objName, directory = Directory, link = False)
	except Exception as E:
		raise Exception ("Problema in import EmptySet.blend: ", E)


def core_importFile():
	global start
	global filePath
	start = time.time()
	bpy.ops.object.select_all(action="DESELECT")
	bpy.context.scene.objects.active = None
	filePath = abspath(bpy.context.scene.BBImportPath)
	id = bpy.context.scene.BBModelRemark
	extension = filePath.lower().endswith
	# check to avoid import models with same name
	if id in modelContainer:
		raise Exception("Error: Duplicate Name Tag.  Try importing again with a different name.")
	# user input sanity check
	if getNumModel() == -1:
		raise Exception("Error: Invalid user ordering of model sequence.  Use comma to separate values")
	if(not bpy.context.scene.objects.get("Empty")):
		create_pdb_empty_set()
	if extension(".pdb") or extension(".pqr"):
		if filePath in importedFiles:
			bpy.ops.object.reimport_file_dialog_operator('INVOKE_DEFAULT')
		else:
			core_parsePDB(filePath)
	if extension(".txt") or extension(".csv"):
		core_parseTXT(filePath)


def core_parsePDB(filePath):
	global currentModelID
	global model
	global modelprog
	global modelList
	global mainChainCache
	global mainChainCache_Nucleic
	global mainChainCache_Nucleic_Filtered
	global chainCache
	global chainCache_Nucleic
	global importedFiles
	modelOrder = [int(model) for model in bpy.context.scene.BBImportOrder.split(',')]
		
	# 2013-06-28 ---  Removes "1+" e "1-"... hoping it will not affect "standard" files...
	try:
		f = open(filePath, "r")
		lines = f.readlines()
		lineCounter = 0
		for line in lines:
			line = line.replace("1+", "  ")
			line = line.replace("1-", "  ")
			lines[lineCounter] = line
			lineCounter = lineCounter + 1
		f.close()
		f = open(filePath, "w")
		f.writelines(lines)
		f.close()
	except Exception as E:
		s = "Unable to fix tmp.pdb: " + str(E)
		print(s)


	# open file (assuming input is valid)
	with open(filePath,"r") as file:
		for line in file:
			line = line.replace("\n", "")
			line = line.replace("\r", "")
			line = PDBString(line)
			tag = line.get("tag")
			# if tag is MODEL, load model ID
			if tag == "MODEL":
				currentModelID = line.get("modelID")
			# if tag is ATOM, load column data (skip this if currentModelID is not in list of models)
			elif (currentModelID in modelOrder) and (tag == "ATOM" or tag == "HETATM"):
				# check for element type
				atomName = line.get("name")
				elementName = line.get("element")
				elementTypeResidue=line.get("aminoName").strip()
				atomtype=line.get("tag")
				# skip water
				if line.get("aminoName") == "HOH":
					continue
				# decide if hydrogen should be skipped
				if not bpy.context.scene.BBImportHydrogen and elementName == H:
					continue
				id = bpy.context.scene.BBModelRemark
				key = str(id) + "#" + line.get("serial").rjust(5, "0")
				model[key] = line
				# add mchain atom data to dictionary for building bonds
				if atomName==N or atomName==C or (atomName==CA and elementName!=CA):
					if key not in mainChainCache: mainChainCache.append(key)
				if atomName in NucleicAtoms:
					if atomName in NucleicAtoms_Filtered:
						if key not in mainChainCache_Nucleic_Filtered: mainChainCache_Nucleic_Filtered.append(key)
					else:
						if key not in mainChainCache_Nucleic: mainChainCache_Nucleic.append(key)
					if atomName =="C3'": mainChainCache_Nucleic.append(key)
				# add all atom data to dictionary for building bonds
				elementTypeNucleic=["D","A","U","G","C","DC","DG","DA","DT"]
				if elementName != H and (not elementTypeResidue in elementTypeNucleic ) and atomtype=="ATOM":
					chainCache[key] = line.get("aminoName") + "#" + line.get("chainSeq") + "#" + line.get("name") + "#" + line.get("chainID") + "#" + line.get("element")
				if elementName != H and (elementTypeResidue in elementTypeNucleic) and atomtype=="ATOM":
					chainCache_Nucleic[key] = line.get("aminoName") + "#" + line.get("chainSeq") + "#" + line.get("name") + "#" + line.get("chainID") + "#" + line.get("element")
			# when the end of a model is reached, add the model dictionary to the list
			if tag == "ENDMDL" or tag=="END" or tag == "MODEL" and (currentModelID in modelOrder):
				if model:
					modelList.append(model)
					modelprog.append(index_element(this_list=modelOrder,elemen_list=currentModelID))
					model = {}
	importedFiles.append(filePath)
	core_sort_hr()


def core_parseTXT(filePath):
	global model
	global modelList
	# Parse text files sequence
	modelOrder = bpy.context.scene.BBImportOrder.split(',')
	for fileName in modelOrder:
		# open each file from the list
		with open(os.path.dirname(filePath)+"/"+fileName.strip(),"r") as f:
			atomCounter = 0
			# force the atom type
			char = C
			# for each line of the file
			for line in f:
				id = bpy.context.scene.BBModelRemark
				key = str(id) + "#" + str(atomCounter).rjust(5, "0")
				line = line.replace('"', "")	# cleanup some unconforming data
				split_line = line.split()
				x = "%8.3f" % float(split_line[0])
				y = "%8.3f" % float(split_line[1])
				z = "%8.3f" % float(split_line[2])
				# a somewhat hackish way to generate a PDB-conformant string for the unified reader to use
				line = PDBString("ATOM                                                                         " + char)
				# insert location in-situ
				line = line.set(30,x)
				line = line.set(38,y)
				line = line.set(46,z)
				model[key] = line
				atomCounter += 1
			modelList.append(model)
			model = {}
	core_sort_hr()


def core_sort_hr():	
	# ======= IMPORT ON SELECTED LAYER = START =======================
	selectedLayer = 0
	tmp = 0
	for x in bpy.context.scene.BBLayerImport:
		if ((x) and (tmp!=19)):
			selectedLayer = tmp
		tmp += 1
	bpy.context.scene.layers[selectedLayer] = True
	for i in range(0,20):
		if (i!=selectedLayer):
			bpy.context.scene.layers[i] = False
	bpy.context.scene.layers[19] = True
	# ======= IMPORT ON SELECTED LAYER = END =========================
	# trying to sort modelList
	global modelList
	try:
		modelList1=[]
		for ikt in modelList:
			modelList1.append(ikt)
		for ikt in range(len(modelprog)):
			modelList1[int(modelprog[ikt])]=modelList[ikt]
		modelList=[]
		for ikt in modelList1:
			modelList.append(ikt)
	except Exception as E:
		raise Exception ("Error sorting model list ", E)
	try:
		if bpy.context.scene.BBSpaceFill:
			objName = "atomFill"
		else:
			objName = "atom"
		Directory = homePath + "library.blend" + os.sep + "Object" + os.sep
		Path = os.sep + os.sep + "library.blend" + os.sep + "Object" + os.sep + objName
		bpy.ops.wm.link_append(filepath = Path, filename = objName, directory = Directory, link = False)
		bpy.data.objects[objName].name ="atom"
		objName = "atom"
		bpy.data.objects[objName].select = True
		bpy.context.scene.objects.active = bpy.data.objects["atom"]
	except Exception as E:
		raise Exception ("Template atom object cannot be loaded from library: ", E)
	# Make high res atom model
	if bpy.context.scene.BBHighQuality:
		bpy.ops.object.modifier_add(type = 'SUBSURF')
		modificatore = bpy.context.scene.objects.active.modifiers[0]
		modificatore.levels = 1
		modificatore.render_levels = 1
		modificatore.name = "SubSurf1"
		bpy.ops.object.modifier_apply(modifier="SubSurf1")
	core_createMaterials()


def core_createMaterials():
	if modelContainer == {}:
		elencoMateriali = [CA, H, N, O, S, ZN, P, FE, MG, MN, CU, NA, K, CL, F]
		bpy.data.materials['atom'].name = C
		bpy.data.materials['C'].diffuse_color = color[C]
		for m in elencoMateriali:
			if not(m in bpy.data.materials):
				bpy.data.materials['C'].copy()
				bpy.data.materials['C.001'].name = m
				bpy.data.materials[m].diffuse_color =  color[m]
	create_fi_materials()
	core_createModels()


def core_createModels():
	global chainCache
	global curFrame
	id = bpy.context.scene.BBModelRemark
	# Build 3D scene from modelList
	for model in modelList:
		# reset
		bpy.ops.object.select_all(action="DESELECT")
		bpy.context.scene.objects.active = None
		bpy.context.scene.frame_set(curFrame)
		# on first model, Place atoms in scene
		if curFrame == 1:
			modelCopy = model.copy()
			# select and temporary rename template atom
			bpy.data.objects["atom"].hide = False
			bpy.data.objects["atom"].select = True
			bpy.data.objects["atom"].name = str(id)
			try:
				# (count - 1) because there is the original template object.
				for j in range(len(model)-1):
					bpy.ops.object.duplicate(linked = True, mode='DUMMY')
			except:
				# We are using a specially patched version of the duplicate operator that is faster
				for i in range(len(model)-1):
					bpy.ops.object.duplicate(linked = True, mode='DUMMY')
			try:
				# walk through list of objects and set name, location and material for each atom
				for i, obj in enumerate(bpy.data.objects):
					# if the object is the one of those we just created (i.e. if name matches xx.0000)
					if obj.name.split(".")[0] == id:
						# descructively walk through the modelCopy varible
						entry = modelCopy.popitem()
						# assign obj name, material, etc.  object Locations are assigned later
						obj.name = entry[0]
						index = entry[1].get("element")
						obj.material_slots[0].material = bpy.data.materials[index]
						# adjust radius
						if bpy.context.scene.BBSpaceFill:
							obj.scale = [scale_vdw[index][0], scale_vdw[index][0], scale_vdw[index][0]]
							obj.game.radius = scale_vdw[index][1]
						else:
							obj.scale = [scale_cov[index][0], scale_cov[index][0], scale_cov[index][0]]
							obj.game.radius = scale_cov[index][1]						
						# add atom info as RNA string to each object
						obj.BBInfo = entry[1][:]
			except Exception as E:
				raise Exception ("Unable to generate 3D model from PDB File", E)
	
			if bpy.context.scene.BBVerbose: print("Build time: " + str(time.time() - start))
			
			if bpy.context.scene.BBMakeBonds:
				try:
					cacheSize = len(mainChainCache) - 1
					for i, entry in enumerate(mainChainCache):
						# Skipping Last Atom to avoid cyclic dependency
						if i < cacheSize:
							# Adding constraints, using atom position to correctly orient hinge x axis
							obj = bpy.data.objects[entry]
							nextEntry = bpy.data.objects[mainChainCache[i+1]]
							line=modelList[0][entry]
							obj.location=line.get("loc")
							line=modelList[0][mainChainCache[i+1]]
							nextEntry.location=line.get("loc")
							addRigidBodyRotamer(obj,nextEntry)
					#bonds for Nucleic Acids
					cacheSize = len(mainChainCache_Nucleic_Filtered) - 1
					for i, entry in enumerate(mainChainCache_Nucleic_Filtered):
						# Skipping Last Atom to avoid cyclic dependency
						if i < cacheSize:
							# Adding constraints, using atom position to correctly orient hinge x axis
							obj = bpy.data.objects[entry]
							nextEntry = bpy.data.objects[mainChainCache_Nucleic_Filtered[i+1]]
							line=modelList[0][entry]
							obj.location=line.get("loc")
							line=modelList[0][mainChainCache_Nucleic_Filtered[i+1]]
							nextEntry.location=line.get("loc")
							addRigidBodyRotamer(obj,nextEntry)	
					if bpy.context.scene.BBVerbose: print("Link time: " + str(time.time() - start))
					chainCache = sorted(chainCache.items())
					for entry in chainCache:
						line = entry[1].split("#")
						amac = line[0]
						chainSeq = line[1]
						atom = line[2]
						chainID = line[3]
						# skip mainchain atoms
						if (atom != C and atom != CA and atom != N and atom != H):
							# for side chain, look up parents based on rules
							parent = bondLookUp(atom=atom, amac=amac)
							# generate name of parents
							target = amac+ "#" + chainSeq + "#" + parent[0] + "#" + chainID + "#" + parent[1]
							# lookup name of blenderobject based on parent name
							targetKey = "atom"
							for item in chainCache:
								if item[1] == target:
									targetKey = item[0]
									break
							# set up the constraint
							# 2013-06-27 --- if targetKey == CA, skip...
							if targetKey == "atom":
								print("TargetKey not set, will skip Rigid Body Joint")
							else:
								obj = bpy.data.objects[entry[0]]
								line=modelList[0][entry[0]]
								obj.location=line.get("loc")
								line=modelList[0][targetKey]
								nextEntry.location=line.get("loc")
								addRigidBodyRotamer(obj,bpy.data.objects[targetKey])
					chainCache = sorted(chainCache_Nucleic.items())
					for entry in chainCache:
						line = entry[1].split("#")
						amac = line[0]
						chainSeq = line[1]
						atom = line[2]
						chainID = line[3]
						# skip mainchain atoms
						#if atom=="O2P" or atom=="O1P" or atom=="C3'" or atom=="C2'"or atom=="C1'" or atom=="O4'":
						if not atom in NucleicAtoms:
							# for side chain, look up parents based on rules
							parent = bondLookUp_NucleicMain(atom=atom, amac=amac)
							# generate name of parents
							target = amac+ "#" + chainSeq + "#" + parent[0] + "#" + chainID + "#" + parent[1]
							# lookup name of blenderobject based on parent name
							targetKey = "atom"
							for item in chainCache:
								if item[1] == target:
									targetKey = item[0]
									break
							# set up the constraint
							obj = bpy.data.objects[entry[0]]
							line=modelList[0][entry[0]]
							obj.location=line.get("loc")
							line=modelList[0][targetKey]
							nextEntry.location=line.get("loc")
							addRigidBodyRotamer(obj,bpy.data.objects[targetKey])	
				except Exception as E:
					raise Exception ("Unable to generate all bonds and constraints:", E)
				if bpy.context.scene.BBVerbose: print("Link time: " + str(time.time() - start))
		try:
			# for all models, insert key frame
			for key, line in model.items():
				OBJ = bpy.data.objects[key]
				OBJ.select = True
				OBJ.location = line.get("loc")
		except Exception as E:
				raise Exception ("Unable to place 3D atoms:", E)
		if len(modelList) != 1:
			# insert keyframe for animations
			bpy.ops.anim.keyframe_insert_menu(type='LocRotScale')
			# advance frame
			bpy.context.scene.frame_end = curFrame
			curFrame += bpy.context.scene.BBDeltaFrame
	if bpy.context.scene.BBVerbose: print("Model animation time: " + str(time.time() - start))
	core_cleaningUp()


def core_cleaningUp():
	# Cleanup
	global activeTag
	global modelContainer
	bpy.context.scene.frame_set(1)
	bpy.ops.object.select_all(action="DESELECT")
	activeTag = id

	# move camera target to center of molecule
	try:
		ob = select("Empty")
		ob.location = atomCentre()
	except Exception as E:
		print("Empty not set to AtomCentre")

	# play animation
	#if (len(modelList) > 1) and not (bpy.context.screen.is_animation_playing):
	#	bpy.ops.screen.animation_play()

	# Finished!  Finally add model to dictionary for multi-model import support
	# for multi-conformation models, we only add the first model to the store.modelContainer
	modelContainer[id] = modelList[0]

	sessionSave()
	
	try:
		bpy.context.scene.objects.active = bpy.data.objects["BioBlender_Camera"]
		bpy.ops.object.constraint_add(type="TRACK_TO")
		bpy.data.objects["BioBlender_Camera"].constraints[0].target = bpy.data.objects["Empty"]
		bpy.data.objects["BioBlender_Camera"].constraints[0].track_axis = 'TRACK_NEGATIVE_Z'
		bpy.data.objects["BioBlender_Camera"].constraints[0].up_axis = 'UP_Y'
	except Exception as E:
		print("BioBlender Camera TrackTo not set")
		
	bpy.context.scene.layers[19] = True
	
	for area in bpy.context.screen.areas:
		if area.type == 'VIEW_3D':
			area.spaces[0].show_relationship_lines = False
	
	try:
		atomAction()
	except Exception as E:
		print("No models, no action")
		
	bpy.ops.object.select_all(action="DESELECT")
	bpy.data.objects['BioBlender_Camera'].select = True
	bpy.context.scene.objects.active = bpy.data.objects['BioBlender_Camera']
	bpy.context.scene.camera = bpy.data.objects["BioBlender_Camera"]
	for area in bpy.context.screen.areas:
		if area.type == 'VIEW_3D':
			area.spaces[0].region_3d.view_perspective="CAMERA"
	
	global firstPDBimported
	firstPDBimported = True
	print ("Finished Importing!")



#I suppose that the object's referential was not change (the same as the scene, with x (red axis) to the right, z (blue axis) up and y (green axis) behing)
def addRigidBodyRotamer(objectparent,objecttarget) : 
	#Add the rigid body joint for rotamer
	#to define a rotamer, an hinge is use, with the axis vector which come from the atom parent to the target and with a position at the center of the parent atom
	
	#This rotation transform the Ox axis of the parent, to the euler angle to orient the x axes (the hinge axis) of the pivot referential from this parent atom to the target  
	parentxaxis = Vector((1.0,0.0,0.0))
	hingevector=Vector((objecttarget.location[0]-objectparent.location[0], objecttarget.location[1]-objectparent.location[1], objecttarget.location[2]-objectparent.location[2]))
	rotvec2mapx2hingevector=parentxaxis.cross(hingevector)	
	rotvec2mapx2hingevector.normalize()
	angle2mapx2hingevector=parentxaxis.angle(hingevector)
	matrot=Matrix.Rotation(angle2mapx2hingevector, 3, rotvec2mapx2hingevector)
	euler=matrot.to_euler()

	#Add the rigid body join for rotamer	
	objectparent.constraints["RigidBody Joint"].target = objecttarget
	#objectparent.constraints["RigidBody Joint"].show_pivot = True
	
	objectparent.constraints["RigidBody Joint"].pivot_x = 0.0
	objectparent.constraints["RigidBody Joint"].pivot_y = 0.0
	objectparent.constraints["RigidBody Joint"].pivot_z = 0.0			
	
	objectparent.constraints["RigidBody Joint"].axis_x = euler[0]
	objectparent.constraints["RigidBody Joint"].axis_y = euler[1]
	objectparent.constraints["RigidBody Joint"].axis_z = euler[2]		
	
	#objectparent.rotation_mode='XYZ'


def atomAction():
	for obj in bpy.data.objects:
		if obj.BBInfo:
			obj.select = True
			bpy.context.scene.objects.active = obj
			actionName = str(obj.name + "Action")
			if actionName in bpy.data.actions.keys():
				obj.game.actuators['F-Curve'].action = bpy.data.actions[actionName]
				obj.game.actuators['F-Curve'].frame_end = bpy.data.actions[actionName].frame_range[1]
			obj.select = False


if __name__ == "__main__":
	print("PDBimport module created")