import bpy
from bpy import *

import mathutils
from mathutils import *
from math import *

from .BB267___COMMONS import *

bpy.types.Object.BBInfo = bpy.props.StringProperty()


viewFilterOld = ""
activeObj = ""
activeobjOld = ""


class bb_view_panel_update(types.Operator):
	bl_idname = "ops.bb_view_panel_update"
	bl_label = "Show Surface"
	bl_description = "Show Surface model"
	def invoke(self, context, event):
		print("invoke surface")
		try:
			if bpy.context.scene.objects.active:
				updateView(residue=bpy.context.scene.objects.active)
			else:
				updateView()
			AtomicMLP(bpy.context.scene.BBAtomicMLP)
		except Exception as E:
			s = "Generate Surface Failed: " + str(E)
			print(s)
			return {'CANCELLED'}
		else:
			return{'FINISHED'}
bpy.utils.register_class(bb_view_panel_update)


# depending on view mode, selectively hide certain object based on atom definition
def updateView(residue = None, verbose = False):
	global modelContainer
	scene = bpy.context.scene
	viewMode = scene.BBViewFilter
	# select amino acid by group
	if residue:
		# skip none atomic object
		if residue.BBInfo:
			seq = PDBString(residue.BBInfo).get("chainSeq")
			id = PDBString(residue.BBInfo).get("chainID")
			for o in bpy.data.objects:
				if(o.BBInfo):
					if((PDBString(o.BBInfo).get("chainSeq") == seq) and (PDBString(o.BBInfo).get("chainID")==id)):
						bpy.data.objects[o.name].select=True
					else:
						bpy.data.objects[o.name].select=False
	# try to select surface
	obj = select("ShapeIndexedFaceSet")
	if viewMode == "4":
		bpy.data.worlds[0].light_settings.use_environment_light = False
		if not obj:
		# generate surface if does not exist
			setup()
			print("first setup made")
			surface()
			print("surface made")
		elif obj.hide:
		# unhide surface if it's hidden
			obj.hide = False
			obj.hide_render = False
		todoAndviewpoints()
	else:
		bpy.data.worlds[0].light_settings.use_environment_light = True
		if obj:
			# hide surface if already exist
			obj.hide = True
			obj.hide_render = True
	for obj in bpy.context.scene.objects:
		obj.hide = False
		obj.hide_render = False
		obj.draw_type = "TEXTURED"
		if re.search("#",obj.name):
			line=obj.BBInfo
			line = PDBString(line)
			elementName = line.get("element")
			atomName = line.get("name")
			# hide all
			if viewMode == "0":
				obj.hide = True
				obj.hide_render = True
			# Main Chain Only
			elif viewMode == "1":
				if not (atomName == N or atomName == C or (atomName == CA and elementName != CA) or (atomName in NucleicAtoms) or (atomName in NucleicAtoms_Filtered)):
					obj.hide = True
					obj.hide_render = True
			# Main Chain and Side Chain Only
			elif viewMode == '2' and elementName ==  H:
				obj.hide = True
				obj.hide_render = True
			# Main Chain and Side Chain Only and H, everything.
			elif viewMode == '4':
				obj.hide = True
				obj.hide_render = True


# Load the saved variables from disk
def sessionLoad(verbose =  False):
	global modelContainer
	global activeTag
	# if the blender file is not saved yet, do nothing
	# if store.modelContainer is empty but there is already a 'large' number of object in the scene:
	if not bpy.data.is_dirty and len(bpy.data.objects) > 500:
		# try to load serialized data from disk or fail silently
		try:
			with open(bpy.data.filepath + ".cache","rb") as filedump:
				modelContainer = pickle.load(filedump)
			select(bpy.data.objects[90].name)    # to select 'something' in the scene
			for model in modelContainer:
				activeTag = model
			if verbose: print("Persistent session loaded")
		except Exception as E:
			print("Warning: Error when loading session cache:", E)


def setup(verbose = False, clear = True):
	global tag
	global modelContainer
	global activeTag
	PDBPathBBImp=bpy.context.scene.BBImportPath#[12:len(bpy.context.scene.BBImportPath)]
	PDBPath = abspath(PDBPathBBImp)
	if clear:
		if linux:
			if os.path.isdir(quotedPath(homePath+"tmp"+ os.sep)):
				shutil.rmtree(quotedPath(homePath+"tmp" + os.sep))
				os.mkdir(quotedPath(homePath+"tmp" + os.sep))
			else:
				os.mkdir(quotedPath(homePath+"tmp" + os.sep))
		else:
			if os.path.isdir(r"\\?\\" + homePath+"tmp"+ os.sep):
				#shutil.rmtree(r"\\?\\" + homePath+"tmp" + os.sep, ignore_errors=True)
				#os.mkdir(r"\\?\\" + homePath+"tmp")
				print("There is a TMP folder!")
			else:
				#os.mkdir(r"\\?\\" + homePath+"tmp" + os.sep)
				print("Trying to making dir on Win (no TMP folder)...")
				os.mkdir(r"\\?\\" + homePath+"tmp")
	if verbose: print("Copying original PDB to tmp directory")
	
	if linux:
		shutil.copy(PDBPath, quotedPath(homePath+"tmp" + os.sep + "original.pdb"))
	else:
		print("Precopy")
		shutil.copy(r"\\?\\" + PDBPath, r"\\?\\" + homePath+"tmp" + os.sep + "original.pdb")
	
	if modelContainer:
		exportPDB(tag = activeTag)
	else:
		print("Warning: Using original PDB, not the one generated by Blender")
		if linux:
			shutil.copy(quotedPath(homePath+"tmp" + os.sep + "original.pdb"), quotedPath(homePath+"tmp" + os.sep + "tmp.pdb"))
		else:
			shutil.copy(r"\\?\\" + homePath+"tmp" + os.sep + "original.pdb", r"\\?\\" + homePath+"tmp" + os.sep + "tmp.pdb")
	# clear zero-user datablocks
	if True:
		for datablock in bpy.data.meshes:
			if datablock.users == 0:
				bpy.data.meshes.remove(datablock)
		for datablock in bpy.data.curves:
				if datablock.users == 0:
					bpy.data.curves.remove(datablock)
		for datablock in bpy.data.objects:
			if datablock.users == 0:
				bpy.data.objects.remove(datablock)
	print("Setup is complete!")


# export scene to PDB file; if no path is specified, it writes to tmp.pdb
def exportPDB(path = homePath+"tmp" + os.sep + "tmp.pdb", tag = None, verbose = False):
	global model
	global modelContainer
	print("=============== exporting PDB")
	print("Exporting model '%s' to %s" %(tag, path))
		
	outPath = abspath(path)
	
	if not tag:	
		for model in modelContainer:
			tag = model
	model = modelContainer[tag]
	ordered = sorted(model.keys())
	print("=======outPath = " + str(outPath))
	with open(outPath, "w") as outFile:
		for obID in ordered:
			try:
				loc = bpy.data.objects[obID].location
				info = bpy.data.objects[obID].BBInfo
			except Exception as E:
				raise Exception ("Error: 3D scene and internal dictionary does not match: ", E)
			x = "%8.3f" % loc[0]
			y = "%8.3f" % loc[1]
			z = "%8.3f" % loc[2]
			# convert line to pdbstring class
			line = PDBString(info)
			# clear location column
			line = line.set(30,"                         ")
			# insert new location
			line = line.set(30,x)
			line = line.set(38,y)
			line = line.set(46,z)
			outFile.write (line+"\n")
		outFile.write("ENDMDL")


# Import the surface generated from PyMol
def surface(verbose = True):
	res = bpy.context.scene.BBMLPSolventRadius
	if select("ShapeIndexedFaceSet"):
		bpy.ops.object.delete()
	if bpy.context.scene.BBHighQuality:
		quality = "1"
	else:
		quality = "0"
	if verbose: print("Generating PyMol script")
	
	# 2013-06-28 -Trying to fix pdb ending with 1- or 1+...
	try:
		oPath = homePath + "tmp" + os.sep + "tmp.pdb"
		f = open(oPath, "r")
		lines = f.readlines()
		lineCounter = 0
		for line in lines:
			line = line.replace("1+", "  ")
			line = line.replace("1-", "  ")
			lines[lineCounter] = line
			lineCounter = lineCounter + 1
		f.close()
		f = open(oPath, "w")
		f.writelines(lines)
		f.close()
	except Exception as E:
		s = "Unable to fix tmp.pdb: " + str(E)
		print(s)
	
	tmpPathO = homePath+"tmp" + os.sep + "surface.pml"
	tmpPathL = "load "+  homePath+"tmp" + os.sep + "tmp.pdb" + "\n"
	tmpPathS = "save " + homePath+"tmp"+os.sep+"tmp.wrl" + "\n"
	
	# 2013-06-28
		#f.write("cmd.move('z',-cmd.get_view()[11])\n")
	
	
	with open(tmpPathO, mode="w") as f:
		f.write("# This file is automatically generated by BioBlender at runtime.\n")
		f.write("# Modifying it manually might not have an effect.\n")
		f.write(tmpPathL)
		f.write('cmd.hide("lines"  ,"tmp")\n')
		f.write('cmd.set("surface_quality"  ,"%s")\n' % quality)
		f.write('cmd.show("surface","tmp")\n')
		f.write('set solvent_radius,'+ str(res) +'\n')
		f.write('cmd.reset()\n')
		f.write('cmd.origin(position=[0,0,0])\n')
		f.write('cmd.center("origin")\n')
		f.write(tmpPathS)
		f.write("quit")
	print("Making Surface using PyMOL")
	
	command = "%s -c -u %s" % (quotedPath(pyMolPath), quotedPath(homePath+"tmp" + os.sep + "surface.pml"))
	
	command = quotedPath(command)
	launch(exeName=command)
	
	bpy.ops.import_scene.x3d(filepath=homePath+"tmp"+os.sep+"tmp.wrl", axis_forward="Y", axis_up="Z")

	ob = bpy.data.objects['ShapeIndexedFaceSet']
	ob.select = True
	bpy.context.scene.objects.active = ob

	bpy.ops.object.editmode_toggle()
	bpy.ops.mesh.remove_doubles(threshold=0.0001, use_unselected=False)
	bpy.ops.object.editmode_toggle()
	bpy.ops.object.shade_smooth()


def AtomicMLP(MLPcolor):
	if MLPcolor:
		# Atomic MLP Color
		for model in modelList:
			modelCopy = model.copy()		
			for i, o in enumerate(bpy.data.objects):
				obj = bpy.data.objects[o.name]
				# if the object is an atom
				if obj.BBInfo:
					aminoName = PDBString(obj.BBInfo).get("aminoName")
					name = PDBString(obj.BBInfo).get("name")
					material_this=retrieve_fi_materials(am_name=aminoName,at_name=name)
					obj.material_slots[0].material = bpy.data.materials[material_this]
		print("Atomic MLP Color set")
	else:
		# Original color
		for model in modelList:
			modelCopy = model.copy()		
			for i, o in enumerate(bpy.data.objects):
				obj = bpy.data.objects[o.name]
				# if the object is an atom
				if obj.BBInfo:
					# In BBInfo, the Atom name is the last string
					index = obj.BBInfo.split()[-1]
					obj.material_slots[0].material = bpy.data.materials[index]
		print("Original Atomic Color set")


if __name__ == "__main__":
	print("VIEW module created")