import bpy
from bpy import *
from urllib.parse import urlencode
from urllib.request import *
from html.parser import *
from smtplib import *
from email.mime.text import MIMEText
import time, platform, os
import codecs
import bpy.path
from bpy.path import abspath
import base64
import mathutils
from mathutils import *
from math import *
import pickle
import shutil
import subprocess


from .BB267___COMMONS import *
from .BB267___GE import *
from .BB267___VIEW import *
from .BB267___PDBimport import *
from .BB267___EP import *
from .BB267___MLP import *
from .BB267___NMA import *
from .BB267___OUTPUT import *


bl_info = {
	"name" : "BioBlender 1.0",
	"author" : "SciVis, IFC-CNR",
	"version" : (1,0),
	"blender" : (2,6,7,0),
	"location" : "Properties Window > Scene",
	"description" : "BioBlender 1.0",
	"warning" : "",
	"wiki_url" : "",
	"tracker_url" : "http://bioblender.eu/?page_id=665",
	"category" : "Add Mesh"
}



# ========================= PDB IMPORT-CORE PANEL block = START =======================
#GUI temp vars
importReady = False
importPathOld =""


class BB_GUI_PDB_IMPORT(bpy.types.Panel):	
	bl_idname = "BB_GUI_PDB_IMPORT"
	bl_label = "BioBlender PDB import"
	bl_space_type = "PROPERTIES"
	bl_region_type = "WINDOW"
	bl_context = "scene"
	bl_options = {'DEFAULT_CLOSED'}
	bpy.types.Scene.BBDeltaFrame = bpy.props.IntProperty(attr="BBDeltaFrame", name="Keyframe Interval", description="The number of in-between frames between each model for animation", default=100, min=1, max=200, soft_min=5, soft_max=50)
	bpy.types.Scene.BBVerbose = bpy.props.BoolProperty(attr="BBVerbose", name="Verbose", description="Be verbose for debugging", default=False)
	bpy.types.Scene.BBSpaceFill = bpy.props.BoolProperty(attr="BBSpaceFill", name="SpaceFill", description="Enable to draw Van der Waals radii, disable to draw covalent radii", default=False)
	bpy.types.Scene.BBSideChain = bpy.props.BoolProperty(attr="BBSideChain", name="SideChain", description="Import sidechain (slower)", default=True)
	bpy.types.Scene.BBImportHydrogen = bpy.props.BoolProperty(attr="BBImportHydrogen", name="Hydrogen", description="Import hydrogen atoms (Slower)", default=False)
	bpy.types.Scene.BBMakeBonds = bpy.props.BoolProperty(attr="BBMakeBonds", name="Make Bonds", description="Generate physic onstrains using atomic bonds (Slower)", default=False)
	bpy.types.Scene.BBHighQuality = bpy.props.BoolProperty(attr="BBHighQuality", name="High Quality", description="Generate high-Poly mesh (Slower)", default=False)
	bpy.types.Scene.BBLayerImport = bpy.props.BoolVectorProperty(size=20, subtype='LAYER', name="Import on Layer", description="Import on Layer")
	bpy.types.Scene.BBImportPath = bpy.props.StringProperty(attr="BBImportPath", description="", default="", subtype="FILE_PATH")
	bpy.types.Scene.BBModelRemark = bpy.props.StringProperty(attr="BBModelRemark", description="Model name tag for multiple imports", default="protein0")
	bpy.types.Scene.BBImportFeedback = bpy.props.StringProperty(attr="BBImportFeedback", description="Import Feedback", default="")
	bpy.types.Scene.BBImportChain = bpy.props.StringProperty(attr="BBImportChain", description="Import Chain", default="")
	bpy.types.Scene.BBImportOrder = bpy.props.StringProperty(attr="BBImportOrder", description="List of models to be imported", default="")
	# ================
	
	def draw(self, context):
		layout = self.layout
		scene = context.scene
		split = layout.split(percentage=0.7)
		split.prop(scene, "BBImportPath", text="")
		split.operator("ops.bb_operator_make_preview")
		row = layout.row()
		row.prop(scene, "BBModelRemark", text="")
		row = layout.row()
		split = layout.split(percentage=0.7)
		col = split.column()
		if getNumModel() != 1:
			col.prop(scene, "BBImportOrder", text="")
		col.prop(scene, "BBImportFeedback", text="", emboss=False)
		col.prop(scene, "BBImportChain", text="", emboss=False)
		row = col.row()
		row.scale_x =0.75
		row.prop(scene, "BBLayerImport")
		row = col.row()
		row.scale_y = 2
		if importReady:
			row.operator("ops.bb_operator_import")
		else:
			row.active = False  
			row.operator("ops.bb_operator_import", text="Error: Not Ready to Import", icon="X")
		# right column
		col = split.column()
		if getNumModel() != 1:
			col.prop(scene, "BBDeltaFrame")
		col.prop(scene, "BBVerbose")
		col.prop(scene, "BBSpaceFill")
		col.prop(scene, "BBImportHydrogen")
		col.prop(scene, "BBMakeBonds")
		col.prop(scene, "BBHighQuality")


class bb_operator_make_preview(types.Operator):
	bl_idname = "ops.bb_operator_make_preview"
	bl_label = "Make Preview"
	bl_description = "Make Preview"
	def invoke(self, context, event):
		try:
			global importReady
			importReady = False
			bpy.context.scene.BBImportFeedback = ""
			bpy.context.scene.BBImportChain = ""
			bpy.context.scene.BBImportOrder = ""
			importReady = importPreview()
		except Exception as E:
			s = "Import Failed: " + str(E)
			print(s)
			return {'CANCELLED'}
		else:
			return{'FINISHED'}
bpy.utils.register_class(bb_operator_make_preview)
# ========================= PDB IMPORT-CORE PANEL block = END =======================


# ========================= VIEW PANEL block = START =======================
class BB_PANEL_VIEW(types.Panel):	
	bl_label = "BioBlender View"
	bl_idname = "BB_PANEL_VIEW"
	bl_space_type = "PROPERTIES"
	bl_region_type = "WINDOW"
	bl_context = "scene"
	bl_options = {'DEFAULT_CLOSED'}
	bpy.types.Object.BBInfo = bpy.props.StringProperty(attr="BBInfo", description="Information on each atom", default="")
	bpy.types.Scene.BBMLPSolventRadius = bpy.props.FloatProperty(attr="BBMLPSolventRadius", name="Solvent Radius", description="Solvent Radius used for Surface Generation", default=1.4, min=0.2, max=5, soft_min=0.4, soft_max=4)
	bpy.types.Scene.BBViewFilter = bpy.props.EnumProperty(attr="BBViewFilter", name="View Filter", description="Select a view mode",
		items=(("1", "Main Chain", ""),
			("2", "+ Side Chain", ""),
			("3", "+ Hydrogen", ""),
			("4", "Surface", "")),
		default="3")
	bpy.types.Scene.BBAtomicMLP = bpy.props.BoolProperty(attr="BBAtomicMLP", name="Atomic MLP", description = "Atomic MLP", default=False)

	def draw(self, context):
		global activeObj
		global activeobjOld
		scene = context.scene
		layout = self.layout
		r = layout.column(align=False)
		if bpy.context.scene.objects.active:
			if(bpy.context.scene.objects.active.BBInfo):
				r.label("Currently Selected Model: " + str(activeObj))
			else:
				r.label("No model selected")
			r.alignment = 'LEFT'
			r.prop(bpy.context.scene.objects.active, "BBInfo", icon="MATERIAL", emboss=False)
		split = layout.split(percentage=0.5)
		r = split.row()
		r.prop(scene, "BBViewFilter", expand=False)
		split = split.row(align=True)
		split.prop(scene, "BBMLPSolventRadius")
		split.prop(scene, "BBAtomicMLP")
		r = layout.row()
		r.operator("ops.bb_view_panel_update", text="APPLY")
	
	@classmethod
	def poll(cls, context):
		global tag
		global activeObj
		global activeobjOld
		try:
			if bpy.context.scene.objects.active.name != None:
				# do a view update when the selected/active obj changes
				if bpy.context.scene.objects.active.name != activeobjOld:
					# get the tag of the active model
					if bpy.context.scene.objects.active.name:
						tag = bpy.context.scene.objects.active.name.split("#")[0]
						# load previous sessions from cache
						if not modelContainer:
							sessionLoad()
							print("Sessionload")
						activeObj = tag
					activeobjOld = bpy.context.scene.objects.active.name
		except Exception as E:
			s = "Context Poll Failed: " + str(E)    # VEEEEEERY ANNOYING...
			#if s.find("BBView")==-1:
			#	print(s)
		return (context)
# ========================= VIEW PANEL block = END =======================


# ========================= GE PANEL block = START =======================
class BB_PHYSICS_SIM_PANEL(types.Panel):	
	bl_label = "BioBlender Game Engine Sim"
	bl_idname = "BB_PHYSICS_SIM_PANEL"
	bl_space_type = "PROPERTIES"
	bl_region_type = "WINDOW"
	bl_context = "scene"
	bl_options = {'DEFAULT_CLOSED'}
	bpy.types.Scene.BBDeltaPhysicRadius = bpy.props.FloatProperty(attr="BBDeltaPhysicRadius", name="Radius", description="Value of radius for collisions in physics engine", default=0.7, min=0.1, max=2, soft_min=.1, soft_max=2)
	bpy.types.Scene.BBSolveMotion = bpy.props.IntProperty(attr="BBSolveMotion", name="Collision", description="Use the physics engine to calculate protein motion (0 is off, 1 is on, 2 is recording)", default=0, min=0, max=2, soft_min=0, soft_max=2)

	def draw(self, context):
		global activeObj
		global activeobjOld
		scene = context.scene
		layout = self.layout
		r = layout.column()
		split = r.split(percentage=0.40)
		split.prop(scene, "BBSolveMotion")
		split.prop(scene, "BBDeltaPhysicRadius")
		split.operator("ops.bb_operator_interactive")
# ========================= GE PANEL block = END =======================


# ========================= EP PANEL block = START =======================
class BB_EP_PANEL(types.Panel):
	bl_label = "BioBlender EP Visualization"
	bl_idname = "BB_EP_PANEL"
	bl_space_type = "PROPERTIES"
	bl_region_type = "WINDOW"
	bl_context = "scene"
	bl_options = {'DEFAULT_CLOSED'}
	bpy.types.Scene.BBForceField = bpy.props.EnumProperty(attr="BBForceField", name="ForceField", description="Select a forcefield type for EP calculation",
	items=(("0", "amber", ""),
	("1", "charmm", ""),
	("2", "parse", ""),
	("3", "tyl06", ""),
	("4", "peoepb", ""),
	("5", "swanson", "")),
	default="0")
	bpy.types.Scene.BBEPIonConc = bpy.props.FloatProperty(attr="BBEPIonConc", name="Ion concentration", description="Ion concentration of the solvent", default=0.15, min=0.01, max=1, soft_min=0.01, soft_max=1)
	bpy.types.Scene.BBEPGridStep = bpy.props.FloatProperty(attr="BBEPGridStep", name="Grid Spacing", description="EP Calculation step size (Smaller is better, but slower)", default=1, min=0.3, max=10, soft_min=0.5, soft_max=5)
	bpy.types.Scene.BBEPMinPot = bpy.props.FloatProperty(attr="BBEPMinPot", name="Minimum Potential", description="Minimum Potential on the surface from which start the calculation of the field lines", default=0.0, min=0.0, max=10000, soft_min=0, soft_max=1000)
	bpy.types.Scene.BBEPNumOfLine = bpy.props.FloatProperty(attr="BBEPNumOfLine", name="n EP Lines*eV/Å² ", description="Concentration of lines", default=0.05, min=0.01, max=0.5, soft_min=0.01, soft_max=0.1, precision=3, step=0.01)
	bpy.types.Scene.BBEPParticleDensity = bpy.props.FloatProperty(attr="BBEPParticleDensity", name="Particle Density", description="Particle Density", default=1, min=0.1, max=10.0, soft_min=0.1, soft_max=5.0)

	def draw(self, context):
		scene = bpy.context.scene
		layout = self.layout
		split = layout.split()
		c = split.column()
		c.prop(scene, "BBForceField")
		c = c.column(align=True)
		c.label("Options:")
		c.prop(scene, "BBEPIonConc")
		c.prop(scene, "BBEPGridStep")
		c.prop(scene, "BBEPMinPot")
		c.prop(scene, "BBEPNumOfLine")
		c.prop(scene, "BBEPParticleDensity")
		c = split.column()
		c.scale_y = 2
		c.operator("ops.bb_operator_ep")
		c.operator("ops.bb_operator_ep_clear")
# ========================= EP PANEL block = START =======================


# ========================= MLP PANEL block = START =====================
class BB_MLP_PANEL(types.Panel):
	bl_label = "BioBlender MLP Visualization"
	bl_idname = "BB_MLP_PANEL"
	bl_space_type = "PROPERTIES"
	bl_region_type = "WINDOW"
	bl_context = "scene"
	bl_options = {'DEFAULT_CLOSED'}
	bpy.types.Scene.BBMLPFormula = bpy.props.EnumProperty(attr="BBMLPFormula", name="Formula", description="Select a formula for MLP calculation", items=(("0", "Dubost", ""), ("1", "Testa", ""), ("2", "Fauchere", ""), ("3", "Brasseur", ""), ("4", "Buckingham", "")), default="1")
	bpy.types.Scene.BBMLPGridSpacing = bpy.props.FloatProperty(attr="BBMLPGridSpacing", name="Grid Spacing", description="MLP Calculation step size (Smaller is better, but slower)", default=1, min=1, max=20, soft_min=1.4, soft_max=10)
	def draw(self, context):
		scene = context.scene
		layout = self.layout
		split = layout.split()
		c = split.column()
		c.prop(scene, "BBMLPFormula")
		c.prop(scene, "BBMLPGridSpacing")
		r = split.row()
		r.scale_y = 2
		r.operator("ops.bb_operator_mlp")
		split = layout.split()
		r = split.column(align=True)
		r = split.column()
		r.scale_y = 2
		r.operator("ops.bb_operator_mlp_render")
# ========================= MLP PANEL block = END =======================


# ========================= NMA PANEL block = START =====================
class BB_NMA_PANEL(types.Panel):
	bl_label = "BioBlender NMA Visualization"
	bl_idname = "BB_NMA_PANEL"
	bl_space_type = "PROPERTIES"
	bl_region_type = "WINDOW"
	bl_context = "scene"
	bl_options = {'DEFAULT_CLOSED'}
	bpy.types.Scene.BBNormalModeAnalysis = bpy.props.EnumProperty(attr="BBNormalModeAnalysis", name="Mode", description="Select a normal mode analysis to show", items=(("0", "1", ""), ("1", "2", ""), ("2", "3", ""), ("3", "4", ""), ("4", "5", ""), ("5", "6", ""), ("6", "7", ""), ("7", "8", ""), ("8", "9", ""), ("9", "10", ""), ("10", "11", ""), ("11", "12", ""), ("12", "13", ""), ("13", "14", ""), ("14", "15", ""), ("15", "16", ""), ("16", "17", ""), ("17", "18", ""), ("18", "19", ""), ("19", "20", "")), default="0")
	bpy.types.Scene.BBNMANbModel = bpy.props.IntProperty(attr="BBNMANbModel", name="NMA steps", description="Number of conformations to be calculated in each direction", default=6, min=1, max=100, soft_min=1, soft_max=50)
	bpy.types.Scene.BBNMARMSD = bpy.props.FloatProperty(attr="BBNMARMSD", name="RMSD sampling", description="RMSD between the given and the farthest conformation", default=0.8, min=0.1, max=5.0, soft_min=0.1, soft_max=5.0, precision=2, step=1.0)
	bpy.types.Scene.BBNMACutoff = bpy.props.FloatProperty(attr="BBNMACutoff", name="NMA cutoff", description="NMA cutoff distance (Å) for pairwise interactions", default=15.0, min=0.0, max=25.0, soft_min=1.0, soft_max=25.0)
	bpy.types.Scene.BBNMAGamma = bpy.props.FloatProperty(attr="BBNMAGamma", name="NMA Gamma", description="NMA spring constant", default=1.0, min=0.0, max=10.0, soft_min=0.1, soft_max=5.0, precision=2, step=1.0)

	def draw(self, context):
		scene = context.scene
		layout = self.layout
		split = layout.split()
		c = split.column()
		c.prop(scene, "BBNormalModeAnalysis")
		c.label("Options:")
		c.prop(scene, "BBNMANbModel")
		c.prop(scene, "BBNMARMSD")
		c.prop(scene, "BBNMACutoff")
		c.prop(scene, "BBNMAGamma")
		c = c.column(align=True)
		c = split.column()
		c.scale_y = 2
		c.operator("ops.bb_operator_nma")
# ============================= NMA PANEL Block = END ================================


# ============================= OUTPUT PANEL Block = START ================================
class BB_OUTPUT_PANEL(types.Panel):
	bl_label = "BioBlender Output"
	bl_idname = "BB_OUTPUT_PANEL"
	bl_space_type = "PROPERTIES"
	bl_region_type = "WINDOW"
	bl_context = "scene"
	bl_options = {'DEFAULT_CLOSED'}
	bpy.types.Scene.BBRender = bpy.props.EnumProperty(attr="BBRender", name="Visualize", description="Select a visualization to render", items=(("0", "Atom", ""), ("1", "Plain Surface", ""), ("2", "MLP", ""), ("3", "EP + Plain Surface", ""), ("4", "EP + MLP", "")), default="4")
	bpy.types.Scene.BBExportStep = bpy.props.IntProperty(attr="BBExportStep", name="Export Step", description="Export step", default=1, min=1, max=100, soft_min=1, soft_max=50)
	def draw(self, context):
		scene = bpy.context.scene
		layout = self.layout
		r = layout.row()
		r.prop(bpy.context.scene.render,"filepath", text="")
		split = layout.split()
		left = split.column()
		left.prop(scene, "BBRender")
		left.prop(bpy.context.scene.render,"stamp_note_text", text="Notes")
		left.prop(bpy.context.scene.render,"use_stamp", text ="Information Overlay")
		left.prop(bpy.context.scene.world.light_settings, "use_environment_light", text="Ambient Light")
		right = split.column()
		c = right.column(align=True)
		c.prop(bpy.context.scene, "frame_start")
		c.prop(bpy.context.scene, "frame_end")
		c.prop(bpy.context.scene, "BBExportStep")
		c1 = right.column()
		c1.scale_y = 2
		c1.operator("ops.bb_operator_export_pdb")
		c1.operator("ops.bb_operator_anim")
		num = ((bpy.context.scene.frame_end - bpy.context.scene.frame_start)/bpy.context.scene.BBExportStep)+1
		c1.label("A total of %d frames will be exported." % num)
# ============================= OUTPUT PANEL Block = END ================================


# ===== REGISTER = START ================================================
def register():
    bpy.utils.register_module(__name__)

def unregister():
    bpy.utils.unregister_module(__name__)

if __name__ == "__main__":
	register()
# ===== REGISTER = END ================================================